import { ParsedOperation, ServiceDescriptor } from '@opas/helper';
import { Schema } from 'dtsgenerator';
export interface ParserOptions {
    /**
     * TypeScript Definitions Namespace
     */
    namespace: string;
    /**
     * OpenAPI schema json url, a http(s) url or file path
     */
    url: string;
    /**
     * post process schema
     */
    postSchema?: (schema: Schema) => PromiseLike<Schema> | Schema;
}
export interface ParserResult extends Record<string, any> {
    schema: Schema;
    definition: string;
    apis: ParsedOperation[];
    service: ServiceDescriptor;
    tags: string[];
}
export default class OpenAPIParser {
    schema: Schema;
    url: string;
    namespace: string;
    postSchema?: (schema: Schema) => PromiseLike<Schema> | Schema;
    constructor(options: ParserOptions);
    /**
     * parse open api json schema
     */
    parse: () => Promise<ParserResult>;
    /**
     * read schema
     * @param url open api json schema url
     * @returns {Promise<Schema>} Schema
     */
    private readSchema;
    /**
     * parse definition from open api json schema
     */
    parseDefinition: () => Promise<string>;
    /**
     * parse apis from open api json schema
     */
    parseApis: () => ParsedOperation[];
    /**
     * parse service info from open api json schema
     */
    parseService: () => ServiceDescriptor;
}
