"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helper_1 = require("@opas/helper");
const dtsgenerator_1 = __importStar(require("dtsgenerator"));
const lodash_1 = require("lodash");
class OpenAPIParser {
    constructor(options) {
        /**
         * parse open api json schema
         */
        this.parse = () => __awaiter(this, void 0, void 0, function* () {
            const schema = yield this.readSchema(this.url);
            const [definition, apis, service] = yield Promise.all([
                this.parseDefinition(),
                this.parseApis(),
                this.parseService(),
            ]);
            const tags = [];
            if (!schema.content.tags) {
                apis.forEach((api) => {
                    tags.push(...(api.tags || []));
                });
                const tagSet = new Set(tags);
                schema.content.tags = Array.from(tagSet).map((tag) => ({
                    name: tag,
                    description: tag,
                }));
            }
            return {
                schema,
                definition,
                apis,
                service,
                tags,
            };
        });
        /**
         * read schema
         * @param url open api json schema url
         * @returns {Promise<Schema>} Schema
         */
        this.readSchema = (url) => __awaiter(this, void 0, void 0, function* () {
            if (/^https?/.test(url)) {
                this.schema = yield (0, dtsgenerator_1.readSchemaFromUrl)(url);
            }
            else {
                const schemas = yield (0, dtsgenerator_1.readSchemasFromFile)(url);
                this.schema = schemas[0];
            }
            // post process schema
            if (this.postSchema) {
                this.schema = yield this.postSchema(this.schema);
            }
            return this.schema;
        });
        /**
         * parse definition from open api json schema
         */
        this.parseDefinition = () => __awaiter(this, void 0, void 0, function* () {
            const content = yield (0, dtsgenerator_1.default)({
                contents: [this.schema],
                config: {
                    plugins: {
                        '@dtsgenerator/replace-namespace': {
                            map: [
                                {
                                    from: [true],
                                    to: [(0, helper_1.pascalCase)(this.namespace)],
                                },
                            ],
                        },
                    },
                },
            });
            return content;
        });
        /**
         * parse apis from open api json schema
         */
        this.parseApis = () => {
            const content = this.schema.content;
            const apis = [];
            const { paths } = content;
            for (const uri in paths) {
                const pathItem = paths[uri];
                for (const method in pathItem) {
                    const operation = pathItem[method];
                    if ('requestBody' in operation && operation.requestBody) {
                        const { requestBody } = operation;
                        operation.parameters = operation.parameters || [];
                        const required = '$ref' in requestBody ? true : requestBody.required;
                        // check is formData
                        if ('content' in requestBody) {
                            Object.entries(requestBody.content).forEach(([mediaType, content]) => {
                                const schema = content.schema;
                                if (mediaType.includes('form-data')) {
                                    operation.parameters.push({
                                        name: 'formData',
                                        in: 'formData',
                                        required,
                                        schema: schema,
                                    });
                                }
                                else if (mediaType.includes('application/x-www-form-urlencoded')) {
                                    operation.parameters.push({
                                        name: 'query',
                                        in: 'query',
                                        required,
                                        schema: schema,
                                    });
                                }
                                else {
                                    operation.parameters.push({
                                        name: 'body',
                                        in: 'body',
                                        required,
                                        schema: schema,
                                    });
                                }
                            });
                        }
                        else {
                            const schema = requestBody;
                            operation.parameters.push({
                                name: 'body',
                                in: 'body',
                                required,
                                schema: schema,
                            });
                        }
                    }
                    let successResponseKey = '200';
                    if (operation.responses) {
                        const keys = Object.keys(operation.responses);
                        const findKey = keys.find((key) => key.startsWith('2'));
                        if (findKey) {
                            successResponseKey = findKey;
                        }
                        else if (!successResponseKey && keys.includes('default')) {
                            successResponseKey = 'default';
                        }
                    }
                    const successResponse = operation.responses[successResponseKey];
                    const parsedOperation = Object.assign(Object.assign({}, operation), { uri,
                        method, operationId: operation.operationId || (0, lodash_1.uniqueId)(), successResponse });
                    apis.push(parsedOperation);
                }
            }
            // sort apis by uri
            return apis.sort((a, b) => a.uri.localeCompare(b.uri));
        };
        /**
         * parse service info from open api json schema
         */
        this.parseService = () => {
            var _a, _b, _c, _d;
            const serviceName = (0, helper_1.camelCase)(this.namespace);
            const service = {
                name: serviceName,
                host: '',
                basePath: '',
                title: '',
                description: '',
                version: '',
            };
            if (this.schema.openApiVersion === 2) {
                const content = this.schema.content;
                service.host = content.host || '';
                service.basePath = content.basePath || '';
                service.title = content.info.title;
                service.description = content.info.description || '';
                service.version = content.info.version || '';
            }
            else {
                const content = this.schema.content;
                service.host = (_b = (_a = content.servers) === null || _a === void 0 ? void 0 : _a[0].url) !== null && _b !== void 0 ? _b : '';
                service.basePath = (_d = (_c = content.servers) === null || _c === void 0 ? void 0 : _c[0].url) !== null && _d !== void 0 ? _d : '';
                service.title = content.info.title;
                service.description = content.info.description || '';
                service.version = content.info.version || '';
            }
            return service;
        };
        this.url = options.url;
        this.namespace = options.namespace;
        this.schema = {};
        this.postSchema = options.postSchema;
    }
}
exports.default = OpenAPIParser;
